"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CssToVariable = void 0;
const glob_1 = require("glob");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const postcss_1 = __importDefault(require("postcss"));
const postcss_scss_1 = __importDefault(require("postcss-scss"));
class CssToVariable {
    constructor(options) {
        this.extractedVariables = [];
        this.variableMap = new Map();
        this.options = {
            directory: options.directory,
            properties: options.properties,
            prefix: options.prefix || 'var',
            outputFile: options.outputFile || 'variables.css',
            pattern: options.pattern || '**/*.{css,scss}',
            nameFormatter: options.nameFormatter || this.defaultNameFormatter.bind(this),
            exportMap: options.exportMap || false
        };
    }
    /**
     * 默认变量命名规则
     */
    defaultNameFormatter(property, value) {
        var _a;
        // 处理SCSS变量
        if (value.startsWith('$')) {
            // 在变量映射中查找实际值
            const scssVarName = value.replace(/^\$/, '');
            const scssVarValue = (_a = this.extractedVariables.find(v => v.variableName === `--${this.options.prefix}-${property}-${scssVarName}`)) === null || _a === void 0 ? void 0 : _a.value;
            if (scssVarValue) {
                const cleanValue = scssVarValue.replace(/^#/, '').replace(/[^a-zA-Z0-9]/g, '');
                return `--${this.options.prefix}-${property}-${cleanValue}`;
            }
            return `--${this.options.prefix}-${property}-${scssVarName}`;
        }
        const cleanValue = value.replace(/^#/, '').replace(/[^a-zA-Z0-9]/g, '');
        return `--${this.options.prefix}-${property}-${cleanValue}`;
    }
    /**
     * 生成变量名
     */
    generateVariableName(property, value) {
        return this.options.nameFormatter(property, value);
    }
    /**
     * 更新变量使用统计
     */
    updateVariableUsage(variable) {
        const key = `${variable.property}:${variable.value}`;
        if (!this.variableMap.has(key)) {
            this.variableMap.set(key, {
                variableName: variable.variableName,
                value: variable.value,
                usageCount: 0,
                usages: []
            });
        }
        const report = this.variableMap.get(key);
        report.usageCount++;
        report.usages.push({
            filePath: variable.filePath,
            line: variable.line,
            property: variable.property,
            value: variable.value
        });
    }
    /**
     * 解析单个文件
     */
    async parseFile(filePath) {
        const content = await fs.promises.readFile(filePath, 'utf-8');
        const result = await (0, postcss_1.default)().process(content, {
            from: filePath,
            syntax: path.extname(filePath) === '.scss' ? postcss_scss_1.default : undefined
        });
        // 创建一个新的PostCSS处理器实例来处理变量替换
        const processor = (0, postcss_1.default)();
        const root = result.root;
        // 首先处理SCSS变量定义
        if (path.extname(filePath) === '.scss') {
            root.walkDecls((decl) => {
                var _a, _b;
                if (decl.prop.startsWith('$')) {
                    const variableName = this.generateVariableName('color', decl.value);
                    const variable = {
                        property: 'color',
                        value: decl.value,
                        variableName,
                        filePath,
                        line: ((_b = (_a = decl.source) === null || _a === void 0 ? void 0 : _a.start) === null || _b === void 0 ? void 0 : _b.line) || 0
                    };
                    this.extractedVariables.push(variable);
                }
            });
        }
        root.walkDecls((decl) => {
            var _a, _b;
            if (this.options.properties.includes(decl.prop)) {
                const variableName = this.generateVariableName(decl.prop, decl.value);
                const variable = {
                    property: decl.prop,
                    value: decl.value,
                    variableName,
                    filePath,
                    line: ((_b = (_a = decl.source) === null || _a === void 0 ? void 0 : _a.start) === null || _b === void 0 ? void 0 : _b.line) || 0
                };
                this.extractedVariables.push(variable);
                this.updateVariableUsage(variable);
                decl.value = `var(${variableName})`;
            }
        });
        const processedResult = await processor.process(root, {
            from: filePath,
            syntax: path.extname(filePath) === '.scss' ? postcss_scss_1.default : undefined
        });
        await fs.promises.writeFile(filePath, processedResult.css);
    }
    /**
     * 生成变量定义文件
     */
    async generateVariablesFile() {
        // 去重变量定义
        const uniqueVariables = new Map();
        for (const variable of this.extractedVariables) {
            uniqueVariables.set(variable.variableName, variable.value);
        }
        const variableDefinitions = Array.from(uniqueVariables.entries())
            .map(([name, value]) => `  ${name}: ${value};`)
            .join('\n');
        const content = `:root {\n${variableDefinitions}\n}\n`;
        const outputPath = path.join(this.options.directory, this.options.outputFile);
        await fs.promises.writeFile(outputPath, content);
        // 导出变量映射关系
        if (this.options.exportMap) {
            const mapOutputPath = outputPath.replace(/\.css$/, '.json');
            const variableReports = Array.from(this.variableMap.values());
            await fs.promises.writeFile(mapOutputPath, JSON.stringify(variableReports, null, 2));
        }
    }
    /**
     * 执行变量提取
     */
    async extract() {
        const files = await (0, glob_1.glob)(this.options.pattern, {
            cwd: this.options.directory,
            absolute: true
        });
        for (const file of files) {
            await this.parseFile(file);
        }
        await this.generateVariablesFile();
    }
    /**
     * 获取变量使用报告
     */
    getVariableReport() {
        return Array.from(this.variableMap.values());
    }
}
exports.CssToVariable = CssToVariable;
